MODULE LCIO4TPC
#define PTRTYPE integer
   USE LCIOapi
   USE TPCdata
   USE Space
   PRIVATE
   
! OpenReadLCIO(CHFILE,IOS)
! ReadLCIOnTbin(nTbin)
! ReadLCIOComment(chComment,chDate,IOS)
! CloseReadLCIO()
   PUBLIC :: OpenReadLCIO,CloseReadLCIO,ReadLCIOComment,ReadLCIOnTbin

! ReadLCIONextEvent(IERR)
   PUBLIC :: ReadLCIONextEvent

! OpenWriteLCIO()
! CloseWriteLCIO()
! WriteLCIOHeader(chComment,chDate)
! WriteLCIOEvent(iflag)
   PUBLIC :: OpenWriteLCIO,CloseWriteLCIO,WriteLCIOHeader,WriteLCIOEvent

   PRIVATE :: ReadLCIOTPChit,ReadLCIOMCtrack

   PTRTYPE,PRIVATE :: reader, writer

CONTAINS
   SUBROUTINE OpenReadLCIO(CHFILE,IOS)
      CHARACTER(LEN=*),INTENT(IN) :: CHFILE
      INTEGER,INTENT(OUT) :: IOS
      INTEGER :: status
      LOGICAL,SAVE :: first=.TRUE.
      
      IF( first )THEN
         reader = lcrdrcreate()
         first = .FALSE.
      ELSE
         status = lcrdrclose( reader )
      ENDIF
      
      status = lcrdropen( reader, CHFILE )
      IF( status == LCIO_SUCCESS )THEN
         IOS = 0
      ELSE
         IOS = 1
      ENDIF
   END SUBROUTINE OpenReadLCIO
   SUBROUTINE ReadLCIOnTbin(nTbin)
      INTEGER,INTENT(OUT) :: nTbin
      PTRTYPE :: event,tphcol,tphit
      INTEGER :: nTb,nTbmin,nTbmax,iHit,nHits,ievent,iflag
      LOGICAL,SAVE :: first=.TRUE.
      
      IF( .not.first )THEN
         nTbin = -1
         RETURN
      ENDIF 
      
      nTbin = -2
      first = .FALSE.
      
      nTbmax = 0
      nTbmin = huge(nTbmin)
      
      event_loop: DO ievent=1,10
         event  = lcrdrreadnextevent(reader,LCIO_READ_ONLY) 
         IF( event==0 ) cycle event_loop
         
         tphcol = lcevtgetcollection(event,LCIO_TPCHIT) 
         IF( tphcol==0 ) cycle event_loop
         
         iflag = lccolgetflag(tphcol)
! BTEST(iflag,LCIO_TPCBIT_RAW): RawData not in LCIO
! BTEST(iflag,1): use Amplitude / Time0 instead
         IF( .NOT.BTEST(iflag,LCIO_TPCBIT_RAW) .OR. BTEST(iflag,1) )THEN
! no raw data stored or use Amplitude/time instead
            nTbin = 0
            RETURN
         ENDIF
         
         nHits = lccolgetnumberofelements( tphcol )
       
         hit_loop: DO iHit=1,nHits
            tphit = lccolgetelementat(tphcol,iHit)
            IF(tphit==0) cycle hit_loop
            nTb  = lctphgetnrawdatawords(tphit)
            nTbmin = min(nTbmin,nTb)
            nTbmax = max(nTbmax,nTb)
         ENDDO hit_loop
         IF( nTbmin==nTbmax )THEN
! let's assume same number of timebins for all events/groups
            nTbin = nTbmax
            RETURN
         ENDIF                  
      ENDDO event_loop

! don't really know how big nTbin can be
      nTbin = max(2*nTbmax,15)
      
   END SUBROUTINE ReadLCIOnTbin
   SUBROUTINE ReadLCIOComment(chComment,chDate,IOS)
      CHARACTER(LEN=*),INTENT(OUT) :: chComment,chDate   
      INTEGER,INTENT(OUT) :: IOS
      PTRTYPE :: runhdr
      
      runhdr = lcrdrreadnextrunheader( reader, LCIO_READ_ONLY )
         
      IF( runhdr /= 0 )THEN
         chComment = lcrhdgetdetectorname( runHdr )
         chDate = lcrhdgetdescription( runHdr )
         IOS = 0
      ELSE
         chComment = " "
         chDate =  " "
         IOS = 1
      ENDIF
      
   END SUBROUTINE ReadLCIOComment

   SUBROUTINE ReadLCIONextEvent(IERR)   
      INTEGER,INTENT(OUT) :: IERR
      PTRTYPE :: event
      
      IERR = 0
      
      event = lcrdrreadnextevent( reader,LCIO_READ_ONLY )
      IF( event==0 ) THEN
         IERR = -1
         RETURN
      ENDIF
      
      RunNumber   = lcevtgetrunnumber(event) 
      EventNumber = lcevtgeteventnumber(event) 
   
      CALL ReadLCIOTPChit(event)
      
      CALL ReadLCIOMCtrack(event)
      
   END SUBROUTINE ReadLCIONextEvent   
 
   SUBROUTINE ReadLCIOTPChit(event)   
      PTRTYPE,INTENT(IN) :: event
      PTRTYPE :: tphcol,tphit
      INTEGER :: iflag,nHits,iHit,iGroup,iP,iPad,nTb,i,rawData
      REAL :: Amplitude,Time0
      
      IADC = 0
      nTbGroup = 0
      
      tphcol = lcevtgetcollection(event, LCIO_TPCHIT )
      IF( tphcol==0 ) RETURN
       
      nHits = lccolgetnumberofelements( tphcol )
      
      iflag = lccolgetflag(tphcol)
      IF(  nTbin>0 .AND. BTEST(iflag,LCIO_TPCBIT_RAW)&
                   .AND. .NOT.BTEST(iflag,1) )THEN
! nTbin ==0: no space for RawData
! BTEST(iflag,LCIO_TPCBIT_RAW): RawData not in LCIO
! BTEST(iflag,1): use Amplitude / Time0 instead

         DO iHit=1,nHits
            tphit = lccolgetelementat(tphcol,iHit)
            IF( tphit==0 ) cycle
             
            iGroup = lctphgetcellid(tphit)   
! in case they are numbered from 0 - nGroups-1
            IF( iGroup == 0 )iGroup = nGroups                
            nTb  = lctphgetnrawdatawords(tphit)
            IF( nTb>nTbin )THEN
               PRINT*,"WARNING, ReadLCIOEvent> time bins of this channel larger than nTbin"
               PRINT*,"EventNumber:",EventNumber," Group:",iGroup," nTb:",nTb," nTbin:",nTbin
               nTb=nTbin
            ENDIF
            
            nTbGroup(iGroup) = nTb
            DO i=1,nTb
! LCIO feature: timebins run from [0,nTb-1]
               rawData = lctphgetrawdataword(tphit,i-1)
               IF( abs(rawData) > huge(IADC(1,1)) )THEN
                  PRINT*,"WARNING, ReadLCIOEvent> IADC=",rawData," >",huge(IADC(1,1))
                  PRINT*,"What FADC do you have? Increase kind for IADC."
                  IF( rawData > 0.0 )THEN
                     rawData = huge(IADC(1,1))
                  ELSE
                     rawData = -huge(IADC(1,1))
                  ENDIF
               ENDIF
               IADC(i,iGroup) = rawData
            ENDDO
         ENDDO
      ENDIF

! Amplitude / Time0         
      DO iPad=1,nPads
         CALL FillSignal(iPad,0.0,1.0)
      ENDDO
      
      DO iHit=1,nHits
         tphit = lccolgetelementat(tphcol,iHit)
         IF( tphit==0 ) cycle
            
         iGroup = lctphgetcellid(tphit)
! in case they are numbered from 0 - nGroups-1
         IF( iGroup == 0 )iGroup = nGroups                
         Amplitude = abs(lctphgcharge(tphit))
         Time0 = lctphgettime(tphit)
         DO iP=1,nPadsinGroup(iGroup)
            iPad = iPadofGroup(iP,iGroup)
            CALL FillSignal(iPad,Amplitude,Time0)
         ENDDO
      ENDDO     
      
   END SUBROUTINE ReadLCIOTPChit
   
   SUBROUTINE ReadLCIOMCtrack(event)
      PTRTYPE,INTENT(IN) :: event
      PTRTYPE :: mccol,pMC
      REAL,DIMENSION(3) :: vertex,momentum
      INTEGER :: nMC
      TYPE(Coordinate_type) :: XY,ZY
      TYPE(Line_type) :: XLine,ZLine

      mccol = lcevtgetcollection(event, LCIO_MCPARTICLE )
      IF( mccol==0 ) RETURN
      
      nMC = lccolgetnumberofelements(mccol)
      IF( nMC == 0 ) RETURN
      
      pMC = lccolgetelementat(mccol,1)
      IF( pMC==0 ) RETURN
      
      vertex = lcmcpgotvertex(pMC)
      momentum = lcmcpgotmomentum(pMC)
      
      XY = Coordinate(vertex(1),vertex(2))
      ZY = Coordinate(vertex(3),vertex(2))
      IF( momentum(2) /= 0.0 )THEN
         MCphi   = momentum(1)/momentum(2)
         MCtheta = momentum(3)/momentum(2)
      ELSE
         MCphi = 0.0
         MCtheta = 0.0
      ENDIF
      
      XLine = Line(XY,MCphi)
      ZLine = Line(ZY,MCtheta)
      MCx0 = getLineX(XLine,0.0)
      MCz0 = getLineX(ZLine,0.0)
       
   END SUBROUTINE ReadLCIOMCtrack
     
   SUBROUTINE CloseReadLCIO()   
      INTEGER :: status
      
      status = lcrdrclose( reader )
      status = lcrdrdelete( reader )
   END SUBROUTINE CloseReadLCIO

! -------------------------------------------------------------------
   
   SUBROUTINE OpenWriteLCIO() 
      INTEGER :: status
      
      writer = lcwrtcreate()
      
      status = lcwrtopen( writer, "thisrun.slcio",LCIO_WRITE_NEW )
      if( status==LCIO_SUCCESS) then
         print*, " opened file .slcio "
      else
         Print*, " could not open file .slcio "
      endif
   END SUBROUTINE OpenWriteLCIO
   SUBROUTINE CloseWriteLCIO() 
      INTEGER :: status
      
      status = lcwrtclose( writer )
      status = lcwrtdelete(writer)
   END SUBROUTINE CloseWriteLCIO
   
   SUBROUTINE WriteLCIOHeader(chComment,chDate)
      CHARACTER(LEN=*),INTENT(IN) :: chComment,chDate   
      PTRTYPE :: runhdr
      INTEGER :: status

      runhdr = lcrhdcreate()
      status = lcrhdsetrunnumber( runhdr, RunNumber )
      status = lcrhdsetdetectorname( runhdr, chComment  )
      status = lcrhdsetdescription( runhdr,chDate)
      status = lcwrtwriterunheader( writer, runhdr )
      status = lcrhddelete(runhdr)
   END SUBROUTINE WriteLCIOHeader
   SUBROUTINE WriteLCIOEvent(iflag)
! iflag describes whether to write RawData and/or DenseData
!     iflag = IBSET(iflag,0) ! RawData   
!     iflag = IBSET(iflag,1) ! DenseData 
      INTEGER, INTENT(IN) :: iflag
      PTRTYPE :: event,tphcol,tphit
      INTEGER :: lcioflag,status,iGroup
      REAL :: Amplitude,T0
      
      event  = lcevtcreate()
      tphcol = lccolcreate( LCIO_TPCHIT )

      status = lcevtsetrunnumber( event, RunNumber ) 
      status = lcevtseteventnumber( event,  EventNumber ) 
      
      lcioflag = 0
! write RawData
      IF( BTEST(iflag,0) ) lcioflag = IBSET(lcioflag,LCIO_TPCBIT_RAW)
! write Amplitude / Time0
      IF( BTEST(iflag,1) ) lcioflag = IBSET(lcioflag,1)

      status = lccolsetflag( tphcol, lcioflag )

      DO iGroup=1,nGroups
         tphit  = lctphcreate() 
         status = lctphsetcellid( tphit, iGroup )          

         IF( BTEST(iflag,0) )THEN
            status = lctphsetrawdata( tphit, INT(IADC(:,iGroup)), nTbin ) 
         ENDIF
         IF( BTEST(iflag,1) )THEN
            CALL getSignalGroup(iGroup,Amplitude,T0)
            status = lctphsetcharge(tphit,Amplitude)
            status = lctphsettime(tphit,T0)
         ENDIF
         status = lccoladdelement(tphcol,tphit) 
      ENDDO
 
!---- now add everything to the event
      status = lcevtaddcollection(event,tphcol,LCIO_TPCHIT)  
      status = lcwrtwriteevent(writer,event)
            
      IF( status == LCIO_ERROR )THEN
         PRINT*, "TPCTEST: error - can not write event", EventNumber
         STOP
      ENDIF
       
      status = lcevtdelete(event)
   END SUBROUTINE WriteLCIOEvent

END MODULE LCIO4TPC
