MODULE LCIOapi
!  adopted for f95 from lciof77api.inc  and  lciof77apiext.inc
!  
!  K.Sachs
!  24/4/2004
!***************************************************
!  Include file for the low level Fortran interface 
!  of LCIO. There is on function for each method of
!  al the relevenat classes in LCIO. Please include
!  this file in your Fortran program and use the
!  C++ API documentation and the naming convention
!  described in the users manual to find the
!  fortran function you need.
!
!  F.Gaede
!  10/30/2003
!**************************************************
#define PTRTYPE integer
  PRIVATE
   
! constants as defined in EVENT::LCIO
  integer,parameter,public :: LCIO_CHBIT_LONG=31,LCIO_CHBIT_BARREL=30, &
                              LCIO_CHBIT_POSZ=29,LCIO_CHBIT_PDG=28
  integer,parameter,public :: LCIO_TPCBIT_RAW=31,LCIO_THBIT_BARREL=31
  integer,parameter,public :: LCIO_READ_ONLY=0,LCIO_UPDATE=1
  integer,parameter,public :: LCIO_WRITE_NEW=0,LCIO_WRITE_APPEND =1
  integer,parameter,public :: LCIO_ERROR=0,LCIO_SUCCESS=1

! collection types:
  character(len=*),parameter,public :: &
  LCIO_LCEVENT           = "LCEvent", &
  LCIO_LCRUNHEADER       = "LCRunHeader", &
  LCIO_MCPARTICLE        = "MCParticle", &
  LCIO_SIMCALORIMETERHIT = "SimCalorimeterHit", &
  LCIO_CALORIMETERHIT    = "CalorimeterHit", &
  LCIO_SIMTRACKERHIT     = "SimTrackerHit", & 
  LCIO_TPCHIT            = "TPCHit", & 
  LCIO_LCFLOATVEC        = "LCFloatVec", &
  LCIO_LCINTVEC          = "LCIntVec"

   
! the LCReader interface
   PUBLIC :: lcrdrcreate,lcrdrdelete,lcrdropen,lcrdrclose
   PUBLIC :: lcrdrreadnextrunheader,lcrdrreadnextevent,lcrdrreadevent
   interface   
      function lcrdrcreate() result(reader)
         PTRTYPE :: reader
      end function lcrdrcreate    
      function lcrdrdelete( reader ) result(status)
         PTRTYPE,intent(in) :: reader
         integer :: status
      end function lcrdrdelete      
      function lcrdropen( reader,file ) result(status)
         PTRTYPE,intent(in) :: reader
         character(len=*),intent(in) :: file
         integer :: status
      end function lcrdropen      
      function lcrdrclose( reader ) result(status)
         PTRTYPE,intent(in) :: reader
         integer :: status
      end function lcrdrclose
      
      function lcrdrreadnextrunheader( reader,accessMode ) result(runhdr)
         PTRTYPE,intent(in) :: reader
         integer,intent(in) :: accessMode
         PTRTYPE :: runhdr
      end function lcrdrreadnextrunheader      
      function lcrdrreadnextevent( reader,accessMode ) result(event)
         PTRTYPE,intent(in) :: reader
         integer,intent(in) :: accessMode
         PTRTYPE :: event
      end function lcrdrreadnextevent      
      function lcrdrreadevent( reader,runNumber,evtNumber ) result(event)
         PTRTYPE,intent(in) :: reader
         integer,intent(in) :: runNumber,evtNumber
         PTRTYPE :: event
      end function lcrdrreadevent      
   end interface

! the LCWriter interface
   PUBLIC :: lcwrtcreate,lcwrtdelete,lcwrtopen,lcwrtclose
   PUBLIC :: lcwrtwriterunheader,lcwrtwriteevent
   interface
      function lcwrtcreate() result(writer)
         PTRTYPE :: writer
      end function lcwrtcreate
      function lcwrtdelete(writer) result(status)
         PTRTYPE,intent(in) :: writer
         integer :: status
      end function lcwrtdelete
      function lcwrtopen( writer,file,writeMode ) result(status)
         PTRTYPE,intent(in) :: writer
         character(len=*),intent(in) :: file
         integer,intent(in) :: writeMode
         integer :: status
      end function lcwrtopen   
      function lcwrtclose( writer ) result(status)
         PTRTYPE,intent(in) :: writer
         integer :: status
      end function lcwrtclose
 
      function lcwrtwriterunheader( writer,runHdr ) result(status)
         PTRTYPE,intent(in) :: writer
         PTRTYPE,intent(in) :: runHdr
         integer :: status
      end function lcwrtwriterunheader
      function lcwrtwriteevent( writer,event ) result(status)
         PTRTYPE,intent(in) :: writer
         PTRTYPE,intent(in) :: event
         integer :: status
      end function lcwrtwriteevent
   end interface
   
! the LCRunHeader interface
   PUBLIC :: lcrhdcreate,lcrhddelete,lcrhdgetrunnumber
   PUBLIC :: lcrhdgetdetectorname,lcrhdgetdescription
   PUBLIC :: lcrhdgetactivesubdetectors
   PUBLIC :: lcrhdsetrunnumber,lcrhdsetdetectorname,lcrhdsetdescription,lcrhdaddactivesubdetector
   interface
      function lcrhdcreate() result(runHdr)
         PTRTYPE :: runHdr
      end function lcrhdcreate
      function lcrhddelete(runHdr) result(status)
         PTRTYPE,intent(in) :: runHdr
         integer :: status
      end function lcrhddelete
      function lcrhdgetrunnumber(runHdr) result(runNumber)
         PTRTYPE,intent(in) :: runHdr
         integer :: runNumber
      end function lcrhdgetrunnumber

      function lcrhdgetdetectorname(runHdr) result(name)
         PTRTYPE,intent(in) :: runHdr
         character(len=80) :: name
      end function lcrhdgetdetectorname
      function lcrhdgetdescription(runHdr) result(name)
         PTRTYPE,intent(in) :: runHdr
         character(len=1024) :: name
      end function lcrhdgetdescription

      function lcrhdgetactivesubdetectors(runHdr) result(sdvec)
         PTRTYPE,intent(in) :: runHdr
         PTRTYPE :: sdvec
      end function lcrhdgetactivesubdetectors
  
      function lcrhdsetrunnumber(runHdr,runNumber) result(status)
         PTRTYPE,intent(in) :: runHdr
         integer,intent(in) :: runNumber
         integer :: status
      end function lcrhdsetrunnumber
      function lcrhdsetdetectorname(runHdr,name) result(status)
         PTRTYPE,intent(in) :: runHdr
         character(len=*),intent(in) :: name
         integer :: status
      end function lcrhdsetdetectorname
      function lcrhdsetdescription(runHdr,name) result(status)
         PTRTYPE,intent(in) :: runHdr
         character(len=*),intent(in) :: name
         integer :: status
      end function lcrhdsetdescription
      function lcrhdaddactivesubdetector(runHdr,name) result(status)
         PTRTYPE,intent(in) :: runHdr
         character(len=*),intent(in) :: name
         integer :: status
      end function lcrhdaddactivesubdetector
   end interface
   
! the LCEvent interface
   PUBLIC :: lcevtcreate,lcevtdelete
   PUBLIC :: lcevtgetcollection,lcevtaddcollection,lcevtremovecollection
   PUBLIC :: lcevtgetrunnumber,lcevtgeteventnumber,lcevtgettimestamp,lcevtgetdetectorname
   PUBLIC :: lcevtsetrunnumber,lcevtseteventnumber,lcevtsettimestamp,lcevtsetdetectorname
   interface
      function lcevtcreate() result(event)
         PTRTYPE :: event
      end function lcevtcreate
      function lcevtdelete(event) result(status)
         PTRTYPE,intent(in) :: event
         integer :: status
      end function lcevtdelete
      
      function lcevtgetcollection(event,colname) result(pcol)
         PTRTYPE,intent(in) :: event
         character(len=*),intent(in) :: colname
         PTRTYPE :: pcol
      end function lcevtgetcollection
      function lcevtaddcollection(event,pcol,colname) result(status)
         PTRTYPE,intent(in) :: event,pcol
         character(len=*),intent(in) :: colname
         integer :: status
      end function lcevtaddcollection
      function lcevtremovecollection(event,colname) result(status)
         PTRTYPE,intent(in) :: event
         character(len=*),intent(in) :: colname
         integer :: status
      end function lcevtremovecollection
      
      function lcevtgetrunnumber(event) result(irun)
         PTRTYPE,intent(in) :: event
         integer :: irun
      end function lcevtgetrunnumber
      function lcevtgeteventnumber(event) result(ievt)
         PTRTYPE,intent(in) :: event
         integer :: ievt
      end function lcevtgeteventnumber
      function lcevtgettimestamp(event) result(itim)
         USE f90_kind
         PTRTYPE,intent(in) :: event
         integer(kind=int64) :: itim
      end function lcevtgettimestamp    
      function lcevtgetdetectorname(event) result(name)
         PTRTYPE,intent(in) :: event
         character(len=80) :: name
      end function lcevtgetdetectorname
      
      function lcevtsetrunnumber(event,irun) result(status)
         PTRTYPE,intent(in) :: event
         integer,intent(in) :: irun
         integer :: status
      end function lcevtsetrunnumber
      function lcevtseteventnumber(event,ievt) result(status)
         PTRTYPE,intent(in) :: event
         integer,intent(in) :: ievt
         integer :: status
      end function lcevtseteventnumber
      function lcevtsetdetectorname(event,name) result(status)
         PTRTYPE,intent(in) :: event
         character(len=*),intent(in) :: name
         integer :: status
      end function lcevtsetdetectorname
      function lcevtsettimestamp(event,itim) result(status)
         USE f90_kind
         PTRTYPE,intent(in) :: event
         integer(kind=int64),intent(in) :: itim
         integer :: status
      end function lcevtsettimestamp
   end interface

! the LCCollection interface
   PUBLIC :: lccolcreate,lccoldelete,lccolsetflag,lccoladdelement,lccolremoveelementat
   PUBLIC :: lccolgettypename,lccolgetnumberofelements,lccolgetelementat,lccolgetflag
   interface
      function lccolcreate(coltype) result(pcol)
         character(len=*),intent(in) :: coltype
         PTRTYPE :: pcol
      end function lccolcreate
      function lccoldelete(pcol) result(status)
         PTRTYPE,intent(in) :: pcol
         integer :: status
      end function lccoldelete
      function lccolsetflag(pcol,iflag) result(status)
         PTRTYPE,intent(in) :: pcol
         integer,intent(in) :: iflag
         integer :: status
      end function lccolsetflag
      function lccoladdelement(pcol,pobject) result(status)
         PTRTYPE,intent(in) :: pcol,pobject
         integer :: status
      end function lccoladdelement
      function lccolremoveelementat(pcol,i) result(status)
         PTRTYPE,intent(in) :: pcol
         integer,intent(in) :: i
         integer :: status
      end function lccolremoveelementat
      
      function lccolgettypename(pcol) result(coltype)
         PTRTYPE,intent(in) :: pcol
         character(len=80) :: coltype
      end function lccolgettypename
      function lccolgetnumberofelements(pcol) result(number)
         PTRTYPE,intent(in) :: pcol
         integer :: number
      end function lccolgetnumberofelements
      function lccolgetelementat(pcol,index) result(pobject)
         PTRTYPE,intent(in) :: pcol
         integer,intent(in) :: index
         PTRTYPE :: pobject
      end function lccolgetelementat
      function lccolgetflag(pcol) result(iflag)
         PTRTYPE,intent(in) :: pcol
         integer :: iflag
      end function lccolgetflag
   end interface

! the MCParticle interface
   PUBLIC :: lcmcpcreate,lcmcpdelete
   PUBLIC :: lcmcpgetnumberofparents,lcmcpgetnumberofdaughters
   PUBLIC :: lcmcpgetparent,lcmcpgetdaughter
   PUBLIC :: lcmcpgetpdg,lcmcpgetgeneratorstatus,lcmcpgetsimulatorstatus
! functions with side effects, define wrapper: get -> got
   PUBLIC :: lcmcpgotendpoint,lcmcpgotvertex,lcmcpgotmomentum
   PUBLIC :: lcmcpgetmass,lcmcpgetcharge,lcmcpgetenergy
   PUBLIC :: lcmcpaddparent,lcmcpsetpdg
   PUBLIC :: lcmcpsetgeneratorstatus,lcmcpsetsimulatorstatus
   PUBLIC :: lcmcpsetvertex,lcmcpsetendpoint,lcmcpsetmomentum
   PUBLIC :: lcmcpsetmass,lcmcpsetcharge
   interface
      function lcmcpcreate() result(pmcp)
         PTRTYPE :: pmcp
      end function lcmcpcreate
      function lcmcpdelete(pmcp) result(status)
         PTRTYPE,intent(in) :: pmcp
         integer :: status
      end function lcmcpdelete
      
      function lcmcpgetnumberofparents(pmcp) result(number)
         PTRTYPE,intent(in) :: pmcp
         integer :: number
      end function lcmcpgetnumberofparents
      function lcmcpgetnumberofdaughters(pmcp) result(number)
         PTRTYPE,intent(in) :: pmcp
         integer :: number
      end function lcmcpgetnumberofdaughters
      function lcmcpgetparent(pmcp,index) result(pmcpp)
         PTRTYPE,intent(in) :: pmcp
         integer,intent(in) :: index
         PTRTYPE :: pmcpp
      end function lcmcpgetparent
      function lcmcpgetdaughter(pmcp,index) result(pmcpd)
         PTRTYPE,intent(in) :: pmcp
         integer,intent(in) :: index
         PTRTYPE :: pmcpd
      end function lcmcpgetdaughter
      
      function lcmcpgetpdg(pmcp,ipdg) result(status)
         PTRTYPE,intent(in) :: pmcp
         integer,intent(in) :: ipdg 
         integer :: status
      end function lcmcpgetpdg
      function lcmcpgetgeneratorstatus(pmcp) result(istatg)
         PTRTYPE,intent(in) :: pmcp
         integer :: istatg
      end function lcmcpgetgeneratorstatus
      function lcmcpgetsimulatorstatus(pmcp) result(istats)
         PTRTYPE,intent(in) :: pmcp
         integer :: istats
      end function lcmcpgetsimulatorstatus
! functions with side effects, define wrapper: get -> got
!      function lcmcpgetendpoint(pmcp,dvtxv) result(status)
!      function lcmcpgetvertex(pmcp,dvtxv) result(status)
!      function lcmcpgetmomentum(pmcp,momv) result(status)
      function lcmcpgetmass(pmcp) result(mass)
         PTRTYPE,intent(in) :: pmcp
         real :: mass
      end function lcmcpgetmass
      function lcmcpgetcharge(pmcp) result(charge)
         PTRTYPE,intent(in) :: pmcp
         real :: charge
      end function lcmcpgetcharge
      function lcmcpgetenergy(pmcp) result(energy)
         PTRTYPE,intent(in) :: pmcp
         real :: energy
      end function lcmcpgetenergy

      function lcmcpaddparent(pmcp,pmcpp) result(status)
         PTRTYPE,intent(in) :: pmcp,pmcpp
         integer :: status
      end function lcmcpaddparent
!      function lcmcpadddaughter(pmcp,pmcpd) result(status)
!         PTRTYPE,intent(in) :: pmcp,pmcpd
!         integer :: status
!      end function lcmcpadddaughter
      function lcmcpsetpdg(pmcp,ipdg) result(status)
         PTRTYPE,intent(in) :: pmcp
         integer,intent(in) :: ipdg
         integer :: status
      end function lcmcpsetpdg
      function lcmcpsetgeneratorstatus(pmcp,istatg) result(status)
         PTRTYPE,intent(in) :: pmcp
         integer,intent(in) :: istatg
         integer :: status
      end function lcmcpsetgeneratorstatus
      function lcmcpsetsimulatorstatus(pmcp,istats) result(status)
         PTRTYPE,intent(in) :: pmcp
         integer,intent(in) :: istats
         integer :: status
      end function lcmcpsetsimulatorstatus
      function lcmcpsetvertex(pmcp,dvtxv) result(status)
         USE f90_kind
         PTRTYPE,intent(in) :: pmcp
         real(KIND=double),dimension(3),intent(in) :: dvtxv 
         integer :: status
      end function lcmcpsetvertex
      function lcmcpsetendpoint(pmcp,dvtxv) result(status)
         USE f90_kind
         PTRTYPE,intent(in) :: pmcp
         real(KIND=double),dimension(3),intent(in) :: dvtxv 
         integer :: status
      end function lcmcpsetendpoint
      function lcmcpsetmomentum(pmcp,momv) result(status)
         PTRTYPE,intent(in) :: pmcp
         real,dimension(3),intent(in) :: momv 
         integer :: status
      end function lcmcpsetmomentum
      function lcmcpsetmass(pmcp,mass) result(status)
         PTRTYPE,intent(in) :: pmcp
         real,intent(in) :: mass
         integer :: status
      end function lcmcpsetmass
      function lcmcpsetcharge(pmcp,charge) result(status)
         PTRTYPE,intent(in) :: pmcp
         real,intent(in) :: charge
         integer :: status
      end function lcmcpsetcharge
   end interface
 
! the SimTrackerHit interface
   PUBLIC :: lcsthcreate,lcsthdelete,lcsthgetmcparticle,lcsthsetmcparticle
   PUBLIC :: lcsthgetcellid,lcsthgetposition,lcsthgetdedx,lcsthgettime
   PUBLIC :: lcsthsetcellid,lcsthsetposition,lcsthsetdedx,lcsthsettime
   interface
      function lcsthcreate() result(pthit)
         PTRTYPE :: pthit
      end function lcsthcreate
      function lcsthdelete(pthit) result(status)
         PTRTYPE,intent(in) :: pthit
         integer :: status
      end function lcsthdelete
      function lcsthgetmcparticle(pthit) result(pmcp)
         PTRTYPE,intent(in) :: pthit
         PTRTYPE :: pmcp
      end function lcsthgetmcparticle
      function lcsthgetcellid(pthit) result(cellid)
         PTRTYPE,intent(in) :: pthit
         integer :: cellid
      end function lcsthgetcellid
      function lcsthgetposition(pthit,index) result(position)
         USE f90_kind
         PTRTYPE,intent(in) :: pthit
         integer,intent(in) :: index
         real(KIND=double) :: position
      end function lcsthgetposition
      function lcsthgetdedx(pthit) result(dedx)
         PTRTYPE,intent(in) :: pthit
         real :: dedx
      end function lcsthgetdedx
      function lcsthgettime(pthit) result(time)
         PTRTYPE,intent(in) :: pthit
         real :: time
      end function lcsthgettime
 
      function lcsthsetcellid(pthit,cellid) result(status)
         PTRTYPE,intent(in) :: pthit
         integer,intent(in) :: cellid
         integer :: status
      end function lcsthsetcellid
      function lcsthsetposition(pthit,posv) result(status)
         USE f90_kind
         PTRTYPE,intent(in) :: pthit
         real(KIND=double),dimension(3),intent(in) :: posv
         integer :: status
      end function lcsthsetposition
      function lcsthsetdedx(pthit,dedx) result(status)
         PTRTYPE,intent(in) :: pthit
         real,intent(in) :: dedx
         integer :: status
      end function lcsthsetdedx
      function lcsthsettime(pthit,time) result(status)
         PTRTYPE,intent(in) :: pthit
         real,intent(in) :: time
         integer :: status
      end function lcsthsettime
      function lcsthsetmcparticle(pthit,pmcp) result(status)
         PTRTYPE,intent(in) :: pthit,pmcp
         integer :: status
      end function lcsthsetmcparticle
   end interface

! the SimCalorimeterHit interface
   PUBLIC :: lcschcreate,lcschdelete,lcschaddmcparticlecontribution
   PUBLIC :: lcschgetcellid0,lcschgetcellid1,lcschgetenergy
   PUBLIC :: lcschgotposition,lcschgetnmcparticles,lcschgetparticlecont
   PUBLIC :: lcschgetenergycont,lcschgettimecont,lcschgetpdgcont
   PUBLIC :: lcschsetcellid0,lcschsetcellid1,lcschsetenergy,lcschsetposition
   interface
      function lcschcreate() result(pchit)
         PTRTYPE :: pchit
      end function lcschcreate
      function lcschdelete(pchit) result(status)
         PTRTYPE,intent(in) :: pchit
         integer :: status
      end function lcschdelete

      function lcschgetcellid0(pchit) result(cellid0)
         PTRTYPE,intent(in) :: pchit
         integer :: cellid0
      end function lcschgetcellid0
      function lcschgetcellid1(pchit) result(cellid1)
         PTRTYPE,intent(in) :: pchit
         integer :: cellid1
      end function lcschgetcellid1
      function lcschgetenergy(pchit) result(energy)
         PTRTYPE,intent(in) :: pchit
         real :: energy
      end function lcschgetenergy
!      function lcschgetposition(pchit,posv) result(status)
      function lcschgetnmcparticles(pchit) result(number)
         PTRTYPE,intent(in) :: pchit
         integer :: number
      end function lcschgetnmcparticles      
      function lcschgetparticlecont(pchit,index) result(pmcp)
         PTRTYPE,intent(in) :: pchit
         integer,intent(in) :: index
         PTRTYPE :: pmcp
      end function lcschgetparticlecont
      function lcschgetenergycont(pchit,index) result(energy)
         PTRTYPE,intent(in) :: pchit
         integer,intent(in) :: index
         real :: energy
      end function lcschgetenergycont
      function lcschgettimecont(pchit,index) result(time)
         PTRTYPE,intent(in) :: pchit
         integer,intent(in) :: index
         real :: time
      end function lcschgettimecont
      function lcschgetpdgcont(pchit,index) result(ipdg)
         PTRTYPE,intent(in) :: pchit
         integer,intent(in) :: index
         integer :: ipdg
      end function lcschgetpdgcont

      function lcschsetcellid0(pchit,cellid0) result(status)
         PTRTYPE,intent(in) :: pchit
         integer,intent(in) :: cellid0
         integer :: status
      end function lcschsetcellid0
      function lcschsetcellid1(pchit,cellid1) result(status)
         PTRTYPE,intent(in) :: pchit
         integer,intent(in) :: cellid1
         integer :: status
      end function lcschsetcellid1
      function lcschsetenergy(pchit,energy) result(status)
         PTRTYPE,intent(in) :: pchit
         real,intent(in) :: energy
         integer :: status
      end function lcschsetenergy
      function lcschsetposition(pchit,posv) result(status)
         PTRTYPE,intent(in) :: pchit
         real,dimension(3),intent(in) :: posv
         integer :: status
      end function lcschsetposition
      function lcschaddmcparticlecontribution(pchit,pmcp,energy,time,ipdg) result(status)
         PTRTYPE,intent(in) :: pchit,pmcp
         real,intent(in) :: energy,time
         integer,intent(in) :: ipdg
         integer :: status
      end function lcschaddmcparticlecontribution
   end interface

! the CalorimeterHit interface
   PUBLIC :: lccahcreate,lccahdelete  
   PUBLIC :: lccahgetcellid0,lccahgetcellid1,lccahgetenergy,lccahgotposition
   PUBLIC :: lccahsetcellid0,lccahsetcellid1,lccahsetenergy,lccahsetposition
   interface
      function lccahcreate() result(pchit)
         PTRTYPE :: pchit
      end function lccahcreate
      function lccahdelete(pchit) result(status)
         PTRTYPE,intent(in) :: pchit
         integer :: status
      end function lccahdelete
      function lccahgetcellid0(pchit) result(cellid0)
         PTRTYPE,intent(in) :: pchit
         integer :: cellid0
      end function lccahgetcellid0
      function lccahgetcellid1(pchit) result(cellid1)
         PTRTYPE,intent(in) :: pchit
         integer :: cellid1
      end function lccahgetcellid1
      function lccahgetenergy(pchit) result(energy)
         PTRTYPE,intent(in) :: pchit
         real :: energy
      end function lccahgetenergy
!      function lccahgotposition(pchit,posv) result(status)

      function lccahsetcellid0(pchit,cellid0) result(status)
         PTRTYPE,intent(in) :: pchit
         integer,intent(in) :: cellid0
         integer :: status
      end function lccahsetcellid0
      function lccahsetcellid1(pchit,cellid1) result(status)
         PTRTYPE,intent(in) :: pchit
         integer,intent(in) :: cellid1
         integer :: status
      end function lccahsetcellid1
      function lccahsetenergy(pchit,energy) result(status)
         PTRTYPE,intent(in) :: pchit
         real,intent(in) :: energy
         integer :: status
      end function lccahsetenergy
      function lccahsetposition(pchit,posv) result(status)
         PTRTYPE,intent(in) :: pchit
         real,dimension(3),intent(in) :: posv
         integer :: status
      end function lccahsetposition
   end interface

! the TPCHit interface
   PUBLIC :: lctphcreate,lctphdelete,lctphsetrawdata
   PUBLIC :: lctphsetcellid,lctphsettime,lctphsetcharge,lctphsetquality
   PUBLIC :: lctphgetcellid,lctphgettime,lctphgcharge,lctphgetquality
   PUBLIC :: lctphgetnrawdatawords,lctphgetrawdataword
   interface
      function lctphcreate() result(pthit)
         PTRTYPE :: pthit
      end function lctphcreate
      function lctphdelete(pthit) result(status)
         PTRTYPE,intent(in) :: pthit
         integer :: status
      end function lctphdelete    
      function lctphsetcellid(pthit,icellid) result(status)
         PTRTYPE,intent(in) :: pthit
         integer,intent(in) :: icellid
         integer :: status
      end function lctphsetcellid
      function lctphsettime(pthit,time) result(status)
         PTRTYPE,intent(in) :: pthit
         real,intent(in) :: time
         integer :: status
      end function lctphsettime
      function lctphsetcharge(pthit,charge) result(status)
         PTRTYPE,intent(in) :: pthit
         real,intent(in) :: charge
         integer :: status
      end function lctphsetcharge
      function lctphsetquality(pthit,quality) result(status)
         PTRTYPE,intent(in) :: pthit
         integer,intent(in) :: quality
         integer :: status
      end function lctphsetquality
      function lctphsetrawdata(pthit,rawData,size) result(status)
         PTRTYPE,intent(in) :: pthit
         integer,intent(in) :: size
         integer,dimension(size),intent(in) :: rawData
         integer :: status
      end function lctphsetrawdata

      function lctphgetcellid(pthit) result(icellid)
         PTRTYPE,intent(in) :: pthit
         integer :: icellid
      end function lctphgetcellid
      function lctphgettime(pthit) result(time)
         PTRTYPE,intent(in) :: pthit
         real :: time
      end function lctphgettime
      function lctphgcharge(pthit) result(charge)
         PTRTYPE,intent(in) :: pthit
         real :: charge
      end function lctphgcharge
      function lctphgetquality(pthit) result(quality)
         PTRTYPE,intent(in) :: pthit
         integer :: quality
      end function lctphgetquality
      
      function lctphgetnrawdatawords(pthit) result(nraw)
         PTRTYPE,intent(in) :: pthit
         integer :: nraw
      end function lctphgetnrawdatawords
      function lctphgetrawdataword(pthit,i) result(iraw)
         PTRTYPE,intent(in) :: pthit
         integer,intent(in) :: i
         integer :: iraw
      end function lctphgetrawdataword
  end interface

! the LCStrVec,LCIntVec,LCFloatVec interface
   PUBLIC :: lcsvcgetlength,lcsvcgetstringat
   PUBLIC :: lcivcgetlength,lcivcgetintat
   PUBLIC :: lcfvcgetlength,lcfvcgetfloatat
   interface
      function lcsvcgetlength(stringvec) result(length)
         PTRTYPE,intent(in) :: stringvec
         integer :: length
      end function lcsvcgetlength
      function lcsvcgetstringat(stringvec,index) result(string)
         PTRTYPE,intent(in) :: stringvec
         integer,intent(in) :: index
         character(len=80) :: string
      end function lcsvcgetstringat
      
      function lcivcgetlength(intvec) result(length)
         PTRTYPE,intent(in) :: intvec
         integer :: length
      end function lcivcgetlength
      function lcivcgetintat(intvec,index) result(i)
         PTRTYPE,intent(in) :: intvec
         integer,intent(in) :: index
         integer :: i
      end function lcivcgetintat
      
      function lcfvcgetlength(floatvec) result(length)
         PTRTYPE,intent(in) :: floatvec
         integer :: length
      end function lcfvcgetlength
      function lcfvcgetfloatat(floatvec,index) result(f)
         PTRTYPE,intent(in) :: floatvec
         integer,intent(in) :: index
         real :: f
      end function lcfvcgetfloatat
   end interface


!************************************************************
!  Include file for the convenient functions of the
!  Fortran interface to LCIO.
!  For documentation purpose the C style function declarations 
!  are provided.
! 
!  F.Gaede
!  10/30/2003
!************************************************************

! --------------- convenient methods to read/write the run header

   PUBLIC :: lcwriterunheader
   interface
!     Write a run header to the specified writer with the given data.
      function lcwriterunheader(writer,irun,detname,descr,sdnamevec,nsubd,nchsubd) result(status)
         PTRTYPE,intent(in) :: writer
         integer,intent(in) :: irun,nsubd,nchsubd
         character(len=*),intent(in) :: detname,descr
         character(len=nchsubd),dimension(nsubd),intent(in) :: sdnamevec
         integer :: status
      end function lcwriterunheader
      
!     Read the next run header and fills the return arguments with the data.
!     PTRTYPE lcreadnextrunheader(PTRTYPE reader, int* irun, void* detname, void* descr,
!     void* sdnamevec, int* nsubd, const int nchsubd) ;
   end interface

! ---------------  convenient methods for the event interface

   PUBLIC :: lcseteventheader,lcdumpevent,lcdumpeventdetailed
   interface
!     Set the complete event header data in the event.
      function lcseteventheader(event,irun,ievent,timestamp,detname) result(status)
         PTRTYPE,intent(in) :: event
         integer,intent(in) :: irun,ievent,timestamp
         character(len=*),intent(in) :: detname
         integer :: status
      end function lcseteventheader

!     Read the complete event header data from the event.
!     int lcgeteventheader(PTRTYPE event, int* irun, int* ievent, int* timestamp, void* detname );

!     Dump the event to the stdout - one line per collection.
      function lcdumpevent(event) result(status)
         PTRTYPE,intent(in) :: event
         integer :: status
      end function lcdumpevent

!     Detailed dump of all the data in the event to stdout.
      function lcdumpeventdetailed(event) result(status)
         PTRTYPE,intent(in) :: event
         integer :: status
      end function lcdumpeventdetailed
   end interface

! ---------------  convenient methods for the mcparticle interface


!     Return all the data of the specified MCParticle in the arguments.
!     int lcgetmcparticledata( PTRTYPE mcparticle, int* pdg, int* hepevtstatus, double* prodvtx,
!     float* momentum, float* mass, float* charge, int* ndaughters ) ;
      
!     Fill the hepevt common block with the MCParicle data in the LCIO event.
!     int lcio2hepevt( PTRTYPE event ) ;

!     Create an MCParticle collection from the hepevt common block and add it to the event.
!     int hepevt2lcio( PTRTYPE event ) ;      

! ---------------  convenient methods for SimTrackerHits

!     Add a new SimTrackerHit with the given data to the collection.
!     int lcaddsimtrackerhit( PTRTYPE col, int cellID, double* pos, float dEdx,
!     float time, PTRTYPE mcp ) ;

!     Return all the data from the specified SimTrackerHit in the arguments.
!     int lcgetsimtrackerhit( PTRTYPE col, int i, int* cellID, double* pos, float* dEdx, 
!     float* time, PTRTYPE* mcp ) ;

! ---------------  convenient methods for SimCalorimeterHits

!     Add a new SimCalorimeterHit with the given data to the collection.
!     Returns a pointer the new hit.
!     PTRTYPE lcaddsimcalohit( PTRTYPE col, int cellID0, int cellID1, float energy, float* pos ) ;

!     Return the data of the specified SimCalorimeterHit in the arguments.
!     int lcgetsimcalohit( PTRTYPE col, int i, int* cellID0, int* cellID1, float* energy, 
!     float* pos ) ;

!     Return the specified contribution of a MCParticle to the hit in the arguments.
!     int lcgetsimcalohitmccont( PTRTYPE hit, int i, PTRTYPE* mcp, float* energy, float* time, 
!     int* pdg ) ;


!----  need to define the HEPEVT common block
! #include "hepevt.inc"
! #include "hepevt1.inc"

CONTAINS

! wrapper for functions with side effects
      function lcmcpgotendpoint(pmcp) result(dvtxv)
         USE f90_kind
         PTRTYPE,intent(in) :: pmcp
         real(KIND=double),dimension(3) :: dvtxv          
         integer :: lcmcpgetendpoint,status
         
         status = lcmcpgetendpoint(pmcp,dvtxv)
         IF( status /= LCIO_SUCCESS ) dvtxv = 0.0
      end function lcmcpgotendpoint
      function lcmcpgotvertex(pmcp) result(dvtxv)
         USE f90_kind
         PTRTYPE,intent(in) :: pmcp
         real(KIND=double),dimension(3) :: dvtxv 
         integer :: lcmcpgetvertex,status
         
         status = lcmcpgetvertex(pmcp,dvtxv)
         IF( status /= LCIO_SUCCESS ) dvtxv = 0.0
      end function lcmcpgotvertex
      function lcmcpgotmomentum(pmcp) result(momv)
         PTRTYPE,intent(in) :: pmcp
         real,dimension(3) :: momv 
         integer :: lcmcpgetmomentum,status
         
         status = lcmcpgetmomentum(pmcp,momv)
         IF( status /= LCIO_SUCCESS ) momv = 0.0
      end function lcmcpgotmomentum
      function lcschgotposition(pchit) result(posv)
         PTRTYPE,intent(in) :: pchit
         real,dimension(3) :: posv
         integer :: lcschgetposition,status
   
         status = lcschgetposition(pchit,posv)
         IF( status /= LCIO_SUCCESS ) posv = 0.0
      end function lcschgotposition
      function lccahgotposition(pchit) result(posv)
         PTRTYPE,intent(in) :: pchit
         real,dimension(3) :: posv
         integer :: lccahgetposition,status
   
         status = lccahgetposition(pchit,posv)
         IF( status /= LCIO_SUCCESS ) posv = 0.0
      end function lccahgotposition

END MODULE LCIOapi



